@@ -107,7 +107,8 @@
 #'   If \code{d} is greater than one and \code{rotate = TRUE} then a rotation
 #'   of the variable axes is performed \emph{after} mode relocation.  The
 #'   rotation is based on the Choleski decomposition (see \link{chol}) of the
-#'   estimated Hessian (computed using \link{optimHess} of the negated
+#'   estimated Hessian (computed using \code{\link[stats]{optimHess}}
+#'   of the negated
 #'   log-density after any user-supplied transformation or Box-Cox
 #'   transformation.  If any of the eigenvalues of the estimated Hessian are
 #'   non-positive (which may indicate that the estimated mode of \code{logf}
@@ -535,15 +536,15 @@ ru <- function(logf, ..., n = 1, d = 1, init = NULL,
   init_psi <- init
   #
   if (trans == "none") {
-    logf_rho <- function(rho, ...) {
-      theta <- rho_to_psi(rho)
+    logf_rho <- function(._rho, ...) {
+      theta <- rho_to_psi(._rho)
       val <- logf(theta, ...) - hscale
       structure(val, theta = theta)
     }
   }
   if (trans == "BC") {
-    logf_rho <- function(rho, ...) {
-      psi <- rho_to_psi(rho)
+    logf_rho <- function(._rho, ...) {
+      psi <- rho_to_psi(._rho)
       # When lambda is not equal to one (psi * const + 1) must be positive
       test <- (psi * const + 1)[which_lam]
       if (any(test <= 0)) return(-Inf)
@@ -555,8 +556,8 @@ ru <- function(logf, ..., n = 1, d = 1, init = NULL,
     }
   }
   if (trans == "user") {
-    logf_rho <- function(rho, ...) {
-      phi <- rho_to_psi(rho)
+    logf_rho <- function(._rho, ...) {
+      phi <- rho_to_psi(._rho)
       theta <- do.call(phi_to_theta, c(list(phi), user_args))
       if (any(!is.finite(theta))) return(-Inf)
       logj <- do.call(log_j, c(list(theta), user_args))
@@ -564,8 +565,8 @@ ru <- function(logf, ..., n = 1, d = 1, init = NULL,
       structure(val, theta = theta)
     }
   }
-  neg_logf_rho <- function(x, ...) -logf_rho(x, ...)
-  f_rho <- function(rho, ...) exp(logf_rho(rho, ...))
+  neg_logf_rho <- function(._rho, ...) -logf_rho(._rho, ...)
+  f_rho <- function(._rho, ...) exp(logf_rho(._rho, ...))
   #
   # Scale logf ---------------------------------
   #
@@ -736,10 +737,10 @@ find_a <-  function(neg_logf_rho, init_psi, d, r, lower, upper, algor,
   #   hessian: the estimated hessian of neg_logf_rho/(d*r+1) at its minimum.
   #
   big_val <- Inf
-  big_finite_val <- 10^10
+  big_finite_val <- 10 ^ 10
   #
   # Function to minimize to find a(r).
-  # Use a weird argument name (._p) to avoid a potential argument-matching
+  # Use a weird argument name (._psi) to avoid a potential argument-matching
   # problem when nlminb() is used.
   a_obj <- function(._psi, ...) {
     # Avoid possible problems with nlminb calling function with NaNs.
@@ -748,34 +749,31 @@ find_a <-  function(neg_logf_rho, init_psi, d, r, lower, upper, algor,
     if (any(is.na(._psi))) return(big_val)
     neg_logf_rho(._psi, ...) / (d * r + 1)
   }
+  # Function for L-BFGS-B and Brent, which don't like Inf or NA
+  a_obj_no_inf <- function(._psi, ...) {
+    check <- neg_logf_rho(._psi, ...) / (d * r + 1)
+    if (is.infinite(check)) {
+      check <- big_finite_val
+    }
+    check
+  }
   #
   if (algor == "optim") {
-    # L-BFGS-B and Brent don't like Inf or NA
-    if (method == "L-BFGS-B" | method == "Brent") {
-      a_obj <- function(._psi, ...) {
-        check <- neg_logf_rho(._psi, ...) / (d * r + 1)
-        if (is.infinite(check)) {
-          check <- big_finite_val
-        }
-        check
-      }
-    }
-    #
     if (method %in% c("L-BFGS-B","Brent")) {
-      temp <- stats::optim(par = init_psi, fn = a_obj, ..., control = control,
-                           hessian = FALSE, method = method, lower = lower,
-                           upper = upper)
+      temp <- stats::optim(par = init_psi, fn = a_obj_no_inf, ...,
+                           control = control, hessian = FALSE, method = method,
+                           lower = lower, upper = upper)
     } else {
       temp <- stats::optim(par = init_psi, fn = a_obj, ..., control = control,
                            hessian = FALSE, method = method)
       # Sometimes Nelder-Mead fails if the initial estimate is too good.
-      # ... so avoid non-zero convergence indicator by using BFGS instead.
+      # ... so avoid non-zero convergence indicator by using L-BFGS-B instead.
       if (temp$convergence == 10) {
-        temp <- stats::optim(par = temp$par, fn = a_obj, ...,
+        temp <- stats::optim(par = temp$par, fn = a_obj_no_inf, ...,
                              control = control, hessian = FALSE,
-                             method = "BFGS")
+                             method = "L-BFGS-B", lower = lower, upper = upper)
       }
-      # In some cases optim with method = "BFGS" may reach it's iteration
+      # In some cases optim with method = "L-BFGS-B" may reach it's iteration
       # limit without the convergence criteria being satisfied.  Then try
       # nlminb as a further check, but don't use the control argument in
       # case of conflict between optim() and nlminb().
@@ -796,10 +794,13 @@ find_a <-  function(neg_logf_rho, init_psi, d, r, lower, upper, algor,
                         lower = lower, upper = upper, control = control)
   # Sometimes nlminb isn't sure that it has found the minimum when in fact
   # it has.  Try to check this, and avoid a non-zero convergence indicator
-  # by using optim with method="BFGS", starting from nlminb's solution.
+  # by using optim with method="L-BFGS-B", starting from nlminb's solution,
+  # but don't use the control argument in case of conflict between
+  # optim() and nlminb().
   if (temp$convergence > 0) {
-    temp <- stats::optim(par = temp$par, fn = a_obj, ..., hessian = FALSE,
-                         method = "BFGS")
+    temp <- stats::optim(par = temp$par, fn = a_obj_no_inf, ...,
+                         hessian = FALSE, method = "L-BFGS-B", lower = lower,
+                         upper = upper)
   }
   # Try to calculate Hessian, but avoid problems if an error is produced.
   # An error may occur if the MAP estimate is very close to a parameter
@@ -870,6 +871,25 @@ find_bs <-  function(f_rho, d, r, lower, upper, f_mode, ep, vals, conv, algor,
     if (f_rho(._rho, ...) == 0) return(big_val)
     -._rho[j] * f_rho(._rho, ...) ^ (r / (d * r + 1))
   }
+  # Functions for L-BFGS-B and Brent, which don't like Inf or NA
+  lower_box_no_inf <- function(._rho, j, ...) {
+    if (any(is.na(._rho))) return(big_finite_val)
+    if (._rho[j] == 0) return(0)
+    if (._rho[j] > 0) return(big_finite_val)
+    if (f_rho(._rho, ...) == 0) return(big_finite_val)
+    check <- ._rho[j] * f_rho(._rho, ...) ^ (r / (d * r + 1))
+    if (is.infinite(check)) check <- big_finite_val
+    check
+  }
+  upper_box_no_inf <- function(._rho, j, ...) {
+    if (any(is.na(._rho))) return(big_finite_val)
+    if (._rho[j] == 0) return(0)
+    if (._rho[j] < 0) return(big_finite_val)
+    if (f_rho(._rho, ...) == 0) return(big_finite_val)
+    check <- -._rho[j] * f_rho(._rho, ...) ^ (r / (d * r + 1))
+    if (is.infinite(check)) check <- big_finite_val
+    check
+  }
   l_box <- u_box <- NULL
   zeros <- rep(0,d)
   #
@@ -890,28 +910,17 @@ find_bs <-  function(f_rho, d, r, lower, upper, f_mode, ep, vals, conv, algor,
       l_box[j] <- temp$objective
       # Sometimes nlminb isn't sure that it has found the minimum when in fact
       # it has.  Try to check this, and avoid a non-zero convergence indicator
-      # by using optim with method="BFGS", starting from nlminb's solution.
+      # by using optim with method="L-BFGS-B", starting from nlminb's solution.
       if (temp$convergence > 0) {
-        temp <- stats::optim(par = temp$par, fn = lower_box, j = j, ...,
-                             hessian = FALSE, method = "BFGS")
+        temp <- stats::optim(par = temp$par, fn = lower_box_no_inf, j = j, ...,
+                             hessian = FALSE, method = "L-BFGS-B",
+                             upper = t_upper, lower = lower - f_mode)
         l_box[j] <- temp$value
       }
     }
     if (algor == "optim") {
-      # L-BFGS-B and Brent don't like Inf or NA
       if (method == "L-BFGS-B" | method == "Brent") {
-        lower_box <- function(._rho, j, ...) {
-          if (any(is.na(._rho))) return(big_finite_val)
-          if (._rho[j] == 0) return(0)
-          if (._rho[j] > 0) return(big_finite_val)
-          if (f_rho(._rho, ...) == 0) return(big_finite_val)
-          check <- ._rho[j] * f_rho(._rho, ...) ^ (r / (d * r + 1))
-          if (is.infinite(check)) check <- big_finite_val
-          check
-        }
-      }
-      if (method == "L-BFGS-B" | method == "Brent") {
-        temp <- stats::optim(par = rho_init, fn = lower_box, j = j, ...,
+        temp <- stats::optim(par = rho_init, fn = lower_box_no_inf, j = j, ...,
                              upper = t_upper, lower = lower - f_mode,
                              control = control, method = method,
                              hessian = FALSE)
@@ -922,11 +931,12 @@ find_bs <-  function(f_rho, d, r, lower, upper, f_mode, ep, vals, conv, algor,
                              hessian = FALSE)
         l_box[j] <- temp$value
         # Sometimes Nelder-Mead fails if the initial estimate is too good.
-        # ... so avoid non-zero convergence indicator by using BFGS instead.
+        # ... so avoid non-zero convergence indicator using L-BFGS-B instead.
         if (temp$convergence == 10) {
-          temp <- stats::optim(par = temp$par, fn = lower_box, j = j, ...,
-                               control = control, method = "BFGS",
-                               hessian = FALSE)
+          temp <- stats::optim(par = temp$par, fn = lower_box_no_inf, j = j,
+                               ..., control = control, method = "L-BFGS-B",
+                               hessian = FALSE,
+                               upper = t_upper, lower = lower - f_mode)
           l_box[j] <- temp$value
         }
         # Check using nlminb() if optim's iteration limit is reached.
@@ -953,28 +963,17 @@ find_bs <-  function(f_rho, d, r, lower, upper, f_mode, ep, vals, conv, algor,
       u_box[j] <- -temp$objective
       # Sometimes nlminb isn't sure that it has found the minimum when in fact
       # it has.  Try to check this, and avoid a non-zero convergence indicator
-      # by using optim with method="BFGS", starting from nlminb's solution.
+      # by using optim with method="L-BFGS-B", starting from nlminb's solution.
       if (temp$convergence > 0) {
-        temp <- stats::optim(par = temp$par, fn = upper_box, j = j, ...,
-                             hessian = FALSE, method = "BFGS")
+        temp <- stats::optim(par = temp$par, fn = upper_box_no_inf, j = j, ...,
+                             hessian = FALSE, method = "L-BFGS-B",
+                             lower = t_lower, upper = upper - f_mode)
         u_box[j] <- -temp$value
       }
     }
     if (algor == "optim") {
-      # L-BFGS-B and Brent don't like Inf or NA
       if (method == "L-BFGS-B" | method == "Brent") {
-        upper_box <- function(._rho, j, ...) {
-          if (any(is.na(._rho))) return(big_finite_val)
-          if (._rho[j] == 0) return(0)
-          if (._rho[j] < 0) return(big_finite_val)
-          if (f_rho(._rho, ...) == 0) return(big_finite_val)
-          check <- -._rho[j] * f_rho(._rho, ...) ^ (r / (d * r + 1))
-          if (is.infinite(check)) check <- big_finite_val
-          check
-        }
-      }
-      if (method == "L-BFGS-B" | method == "Brent") {
-        temp <- stats::optim(par = rho_init, fn = upper_box, j = j, ...,
+        temp <- stats::optim(par = rho_init, fn = upper_box_no_inf, j = j, ...,
                              lower = t_lower, upper = upper - f_mode,
                              control = control, method = method)
         u_box[j] <- -temp$value
@@ -983,10 +982,11 @@ find_bs <-  function(f_rho, d, r, lower, upper, f_mode, ep, vals, conv, algor,
                              control = control, method = method)
         u_box[j] <- -temp$value
         # Sometimes Nelder-Mead fails if the initial estimate is too good.
-        # ... so avoid non-zero convergence indicator by using BFGS instead.
+        # ... so avoid non-zero convergence indicator using L-BFGS-B instead.
         if (temp$convergence == 10) {
-          temp <- stats::optim(par = temp$par, fn = upper_box, j = j, ...,
-                               control = control, method = "BFGS")
+          temp <- stats::optim(par = temp$par, fn = upper_box_no_inf, j = j,
+                               ..., control = control, method = "L-BFGS-B",
+                               lower = t_lower, upper = upper - f_mode)
           u_box[j] <- -temp$value
         }
         # Check using nlminb() if optim's iteration limit is reached.
